// Trigger.cpp : Implementation of CTrigger
#include "stdafx.h"
#include "KeckTrigger.h"
#include "Trigger.h"

#include <olmem.h>         
#include <olerrors.h>         
#include <oldaapi.h>

#define CHECKERROR(ecode) if ((status = (ecode)) != OLNOERROR)\
                  {\
                  olDaReleaseDASS(ct_ss);\
                  olDaTerminate(hdrvr);\
                  return status;}

HDEV hdrvr;
ECODE status;
HDASS ct_ss;
HDASS ct_ss2;

BOOL m_Trigger;
long m_FrameRate;

STDMETHODIMP CTrigger::SetRate(long rate)
{
	m_FrameRate = rate;
	if (m_Trigger)
	{
		StopTrigger();
		StartTrigger();
	}

	return S_OK;
}

STDMETHODIMP CTrigger::StartTrigger()
{
	if (!m_Trigger)
	{
		// Get first available Open Layers board 
		CHECKERROR (olDaInitialize("Board1", &hdrvr));

		// get handle to CT sub system 
		CHECKERROR (olDaGetDASS(hdrvr, OLSS_CT, 4, &ct_ss));
		CHECKERROR (olDaGetDASS(hdrvr, OLSS_CT, 6, &ct_ss2));

		// setup internally cascaded clock
		olDaSetCascadeMode(ct_ss, OL_CT_CASCADE);
		olDaSetCascadeMode(ct_ss2, OL_CT_CASCADE);

		// specify internal CT clock input source
		CHECKERROR (olDaSetClockSource(ct_ss, OL_CLK_INTERNAL));
		CHECKERROR (olDaSetClockSource(ct_ss2, OL_CLK_INTERNAL));

		// specify gate type
		CHECKERROR (olDaSetGateType(ct_ss, OL_GATE_NONE));
		CHECKERROR (olDaSetGateType(ct_ss2, OL_GATE_NONE));

		// specify frequency of CT clock output
		CHECKERROR (olDaSetClockFrequency(ct_ss, (double)m_FrameRate));
		CHECKERROR (olDaSetClockFrequency(ct_ss2, (double)m_FrameRate));

		// setup for rate generation
		CHECKERROR (olDaSetCTMode(ct_ss, OL_CTMODE_RATE));
		CHECKERROR (olDaSetCTMode(ct_ss2, OL_CTMODE_RATE));

		// specify polarity of output pulses
		CHECKERROR (olDaSetPulseType(ct_ss, OL_PLS_HIGH2LOW));
		CHECKERROR (olDaSetPulseType(ct_ss2, OL_PLS_HIGH2LOW));

		// specify pulse width
		CHECKERROR (olDaSetPulseWidth(ct_ss, 95.00));
		CHECKERROR (olDaSetPulseWidth(ct_ss2, 95.00));

		// configure the system
		CHECKERROR (olDaConfig(ct_ss));
		CHECKERROR (olDaConfig(ct_ss2));

		// start signal generator
		CHECKERROR (olDaStart(ct_ss));
		CHECKERROR (olDaStart(ct_ss2));
		m_Trigger = true;
	}
	return S_OK;
}

STDMETHODIMP CTrigger::StopTrigger()
{
	if (m_Trigger)
	{
		// stop signal generator
		CHECKERROR (olDaStop(ct_ss));
		CHECKERROR (olDaStop(ct_ss2));

		// Release handle to CT sub system 
		CHECKERROR (olDaReleaseDASS(ct_ss));
		CHECKERROR (olDaReleaseDASS(ct_ss2));

		// Release first available Open Layers board 
		CHECKERROR (olDaTerminate(hdrvr));
		m_Trigger = false;
	}
	return S_OK;
}

STDMETHODIMP CTrigger::GetStatus(BOOL *status)
{
	*status = m_Trigger;
	return S_OK;
}

STDMETHODIMP CTrigger::Beep(long Freq, long Len)
{
	::Beep(Freq,Len);
	return S_OK;
}

void InitTrigger()
{
	m_FrameRate = 30;
	m_Trigger = false;
}

void CloseTrigger()
{
	if (m_Trigger)
	{
		// stop signal generator
		olDaStop(ct_ss);
		olDaStop(ct_ss2);

		// Release handle to CT sub system 
		olDaReleaseDASS(ct_ss);
		olDaReleaseDASS(ct_ss2);

		// Release first available Open Layers board 
		olDaTerminate(hdrvr);
		m_Trigger = false;
	}
}

